/*
 * Copyright (c) 2019 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include "retained.h"

#include <inttypes.h>
#include <stdio.h>

#include <zephyr/device.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/kernel.h>
#include <zephyr/pm/device.h>
#include <zephyr/sys/poweroff.h>
#include <zephyr/sys/util.h>

static const struct gpio_dt_spec sw0 = GPIO_DT_SPEC_GET(DT_ALIAS(sw0), gpios);
static const struct gpio_dt_spec sw1 = GPIO_DT_SPEC_GET(DT_ALIAS(sw1), gpios);
static const struct gpio_dt_spec sw2 = GPIO_DT_SPEC_GET(DT_ALIAS(sw2), gpios);
static const struct gpio_dt_spec sw3 = GPIO_DT_SPEC_GET(DT_ALIAS(sw3), gpios);

static const struct gpio_dt_spec doci_pin = {
                                                .port = DEVICE_DT_GET(DT_NODELABEL(gpio0)),
                                                .pin = 26,
                                                .dt_flags = (GPIO_PULL_DOWN | GPIO_ACTIVE_HIGH)
                                             };

int main(void)
{
	int rc;
	const struct device *const cons = DEVICE_DT_GET(DT_CHOSEN(zephyr_console));

	if (!device_is_ready(cons)) {
		printf("%s: device not ready.\n", cons->name);
		return 0;
	}

	printf("\n%s system off demo\n", CONFIG_BOARD);

#if 0
	if (IS_ENABLED(CONFIG_APP_RETENTION)) {
		bool retained_ok = retained_validate();

		/* Increment for this boot attempt and update. */
		retained.boots += 1;
		retained_update();

		printf("Retained data: %s\n", retained_ok ? "valid" : "INVALID");
		printf("Boot count: %u\n", retained.boots);
		printf("Off count: %u\n", retained.off_count);
		printf("Active Ticks: %" PRIu64 "\n", retained.uptime_sum);
	} else {
		printf("Retained data not supported\n");
	}
#endif

	/* configure sw0 as input, interrupt as level active to allow wake-up */
	rc = gpio_pin_configure_dt(&sw0, GPIO_INPUT);
	if (rc < 0) {
		printf("Could not configure sw0 GPIO (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure sw0 GPIO (%d)\n", rc);
   }

	rc = gpio_pin_configure_dt(&sw1, GPIO_INPUT);
	if (rc < 0) {
		printf("Could not configure sw1 GPIO (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure sw1 GPIO (%d)\n", rc);
   }

	rc = gpio_pin_configure_dt(&sw2, GPIO_INPUT);
	if (rc < 0) {
		printf("Could not configure sw2 GPIO (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure sw2 GPIO (%d)\n", rc);
   }

	rc = gpio_pin_configure_dt(&sw3, GPIO_INPUT);
	if (rc < 0) {
		printf("Could not configure sw3 GPIO (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure sw3 GPIO (%d)\n", rc);
   }

   rc = gpio_pin_configure_dt(&doci_pin, GPIO_INPUT);
	if (rc < 0) {
		printf("Could not config doci_pin GPIO (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure doci_pin GPIO (%d)\n", rc);
   }

	rc = gpio_pin_interrupt_configure_dt(&sw0, GPIO_INT_EDGE_TO_INACTIVE);
	if (rc < 0) {
		printf("Could not configure sw0 GPIO interrupt (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure sw0 GPIO interrupt (%d)\n", rc);
   }

	rc = gpio_pin_interrupt_configure_dt(&sw1, GPIO_INT_EDGE_TO_INACTIVE);
	if (rc < 0) {
		printf("Could not configure sw1 GPIO interrupt (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure sw1 GPIO interrupt (%d)\n", rc);
   }

	rc = gpio_pin_interrupt_configure_dt(&sw2, GPIO_INT_EDGE_TO_INACTIVE);
	if (rc < 0) {
		printf("Could not configure sw2 GPIO interrupt (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure sw2 GPIO interrupt (%d)\n", rc);
   }

	rc = gpio_pin_interrupt_configure_dt(&sw3, GPIO_INT_EDGE_TO_INACTIVE);
	if (rc < 0) {
		printf("Could not configure sw3 GPIO interrupt (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure sw3 GPIO interrupt (%d)\n", rc);
   }
#if 1
   rc = gpio_pin_interrupt_configure_dt(&doci_pin, GPIO_INT_EDGE_TO_ACTIVE);
	if (rc < 0) {
		printf("Could not configure doci_pin GPIO interrupt (%d)\n", rc);
		return 0;
	}
   else {
		printf("configure doci_pin GPIO interrupt (%d)\n", rc);
   }
#endif
	printf("Entering system off; press sw0, sw1, sw2, sw3 to restart\n");

#if 0
	rc = pm_device_action_run(cons, PM_DEVICE_ACTION_SUSPEND);
	if (rc < 0) {
		printf("Could not suspend console (%d)\n", rc);
		return 0;
	}

	if (IS_ENABLED(CONFIG_APP_RETENTION)) {
		/* Update the retained state */
		retained.off_count += 1;
		retained_update();
	}
#endif

	sys_poweroff();
	printf("#### Entered system off; press sw0, sw1, sw2, sw3 to restart\n");

	return 0;
}
